@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors:
  *   Naba Kumar <naba@gnome.org>
@@ -82,6 +82,7 @@ struct _GladeAppPriv {
 				      * GladeWidgets.
 				      */
 	GtkAccelGroup *accel_group;	/* Default acceleration group for this app */
+	GList *undo_list, *redo_list;	/* Lists of buttons to refresh in update-ui signal */
 };
 
 enum
@@ -149,6 +150,51 @@ glade_app_finalize (GObject *app)
 		parent_class->finalize (app);
 }
 
+static void
+glade_app_refresh_undo_redo_button (GladeApp *app,
+				    GtkWidget *button,
+				    gboolean undo)
+{
+	GladeCommand *command = NULL;
+	static GtkTooltips *button_tips = NULL;
+	GladeProject *project;
+	gchar *desc;
+
+	if (button_tips == NULL)
+		button_tips = gtk_tooltips_new ();
+	
+	if ((project = glade_app_get_active_project (app)) != NULL)
+	{
+		if (undo)
+			command = glade_command_next_undo_item (project);
+		else
+			command = glade_command_next_redo_item (project);
+	}
+
+	/* Change tooltips */
+	desc = g_strdup_printf ((undo) ? _("Undo: %s") : _("Redo: %s"),
+			command ? command->description : _("the last action"));
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (button_tips), button, desc, NULL);
+	g_free (desc);
+
+	/* Set sensitivity on the button */
+	gtk_widget_set_sensitive (button, command != NULL);
+}
+
+void
+glade_app_update_ui_default (GladeApp *app)
+{
+	GList *list;
+	
+	for (list = app->priv->undo_list; list; list = list->next)
+		if (list->data)
+			glade_app_refresh_undo_redo_button (app, list->data, TRUE);
+
+	for (list = app->priv->redo_list; list; list = list->next)
+		if (list->data)
+			glade_app_refresh_undo_redo_button (app, list->data, FALSE);
+}
+
 static void
 glade_app_class_init (GladeAppClass * klass)
 {
@@ -161,7 +207,7 @@ glade_app_class_init (GladeAppClass * klass)
 	object_class->dispose   = glade_app_dispose;
 	object_class->finalize  = glade_app_finalize;
 
-	klass->update_ui_signal = NULL;
+	klass->update_ui_signal = glade_app_update_ui_default;
 	klass->show_properties  = NULL;
 	klass->hide_properties  = NULL;
 
@@ -382,6 +428,7 @@ glade_app_init (GladeApp *app)
 	g_object_ref (app->priv->editor);
 	gtk_object_sink (GTK_OBJECT (app->priv->editor));
 	gtk_widget_show_all (GTK_WIDGET (app->priv->editor));
+	glade_editor_refresh (app->priv->editor);
 	
 	/* Create clipboard */
 	app->priv->clipboard = glade_clipboard_new ();
@@ -393,6 +440,9 @@ glade_app_init (GladeApp *app)
 
 	/* Load the configuration file */
 	app->priv->config = glade_app_config_load (app);
+	
+	/* Undo/Redo button list */
+	app->priv->undo_list = app->priv->redo_list = NULL;
 }
 
 GType
@@ -529,7 +579,11 @@ on_project_selection_changed_cb (GladeProject *project, GladeApp *app)
 	g_return_if_fail (GLADE_IS_PROJECT (project));
 	g_return_if_fail (GLADE_IS_APP (app));
 
-	if (app->priv->editor)
+	/* Only update the editor if the selection has changed on
+	 * the currently active project.
+	 */
+	if (app->priv->editor &&
+	    (project == glade_app_get_active_project (app)))
 	{
 		list = glade_project_selection_get (project);
 		num = g_list_length (list);
@@ -554,8 +608,8 @@ glade_app_is_project_loaded (GladeApp *app, const gchar *project_path)
 	{
 		GladeProject *cur_project = GLADE_PROJECT (list->data);
 
-		if ((loaded = glade_util_basenames_match (cur_project->path, 
-							  project_path)))
+		if ((loaded = cur_project->path && 
+		     (strcmp (cur_project->path, project_path) == 0)))
 			break;
 	}
 
@@ -585,6 +639,27 @@ glade_app_hide_properties (GladeApp* app)
 		g_critical ("%s not implemented\n", G_GNUC_FUNCTION);
 }
 
+void
+glade_app_update_instance_count (GladeApp *app, GladeProject *project)
+{
+	GladeProject *prj;
+	GList *l;
+	gint temp, max = 0, i = 0;
+		
+	for (l = app->priv->projects; l; l = l->next)
+	{
+		prj = l->data;
+
+		if (prj != project &&
+		    prj->name && !strcmp (prj->name, project->name))
+		{
+			i++;
+			temp = MAX (prj->instance + 1, i);
+			max  = MAX (temp, max);
+		}
+	}
+	project->instance = MAX (max, i);
+}
 
 void
 glade_app_add_project (GladeApp *app, GladeProject *project)
@@ -597,8 +672,10 @@ glade_app_add_project (GladeApp *app, GladeProject *project)
 		glade_app_set_project (app, project);
 		return;
 	}
-	
- 	app->priv->projects = g_list_prepend (app->priv->projects, project);
+
+	glade_app_update_instance_count (app, project);
+
+	app->priv->projects = g_list_prepend (app->priv->projects, project);
 	
 	/* connect to the project signals so that the editor can be updated */
 	g_signal_connect (G_OBJECT (project), "widget_name_changed",
@@ -618,8 +695,6 @@ glade_app_add_project (GladeApp *app, GladeProject *project)
 void
 glade_app_remove_project (GladeApp *app, GladeProject *project)
 {
-	GList *list;
-
 	g_return_if_fail (GLADE_IS_APP (app));
 	g_return_if_fail (GLADE_IS_PROJECT (project));
 
@@ -632,22 +707,15 @@ glade_app_remove_project (GladeApp *app, GladeProject *project)
 	/* If no more projects */
 	if (app->priv->projects == NULL)
 	{
-		for (list = app->priv->views; list; list = list->next)
-		{
-			GladeProjectView *view;
-
-			view = GLADE_PROJECT_VIEW (list->data);
-			glade_project_view_set_project (view, NULL);
-		}
-		glade_editor_load_widget (app->priv->editor, NULL);
+		/* XXX I think the palette should detect this. */
 		gtk_widget_set_sensitive (GTK_WIDGET (app->priv->palette), FALSE);
 	} 
 	else
 		glade_app_set_project (app, app->priv->projects->data);
 
-	/* Release project after disconnecting from view, so that
-	 * the glade-project-view has a project to g_signal_handler_disconnect()
-	 * from (without bothering with the trickyness of weak pointers and such).
+	/* Its safe to just release the project as the project emits a
+	 * "close" signal and everyone is responsable for cleaning up at
+	 * that point.
 	 */
 	g_object_unref (project);
 }
@@ -715,10 +783,14 @@ glade_app_command_paste (GladeApp *app)
 {
 	if (app->priv->active_project)
 	{
-		GList       *list   = glade_project_selection_get (app->priv->active_project);
-		GladeWidget *parent = list ? glade_widget_get_from_gobject (list->data) : NULL;
+		GList         *list   = glade_project_selection_get (app->priv->active_project);
+		GladePlaceholder *placeholder = list ? NULL : glade_util_selected_placeholder ();
+		GladeWidget   *parent =
+			list ? glade_widget_get_from_gobject (list->data) :
+			glade_placeholder_get_parent (placeholder);
+		
+		glade_util_paste_clipboard (placeholder, parent);
 
-		glade_util_paste_clipboard (NULL, parent);
 		/* Update UI. */
 		glade_app_update_ui (app);
 	}
@@ -774,15 +846,86 @@ glade_app_set_accel_group (GladeApp *app, GtkAccelGroup *accel_group)
 	GladeProject *project;
 	g_return_if_fail(GLADE_IS_APP(app) &&
 			 GTK_IS_ACCEL_GROUP (accel_group));
-	
-	app->priv->accel_group = accel_group;
 
 	for (l = app->priv->projects; l; l = l->next)
 	{
 		project = l->data;
-		glade_project_set_accel_group
-			(project, app->priv->accel_group);
+		glade_project_set_accel_group (project, accel_group);
 	}
+	
+	app->priv->accel_group = accel_group;
+}
+
+static gboolean
+glade_app_undo_button_destroyed (GtkWidget *button, GladeApp *app)
+{
+	app->priv->undo_list = g_list_remove (app->priv->undo_list, button);
+	return FALSE;
+}
+
+static gboolean
+glade_app_redo_button_destroyed (GtkWidget *button, GladeApp *app)
+{
+	app->priv->redo_list = g_list_remove (app->priv->redo_list, button);
+	return FALSE;
+}
+
+static GtkWidget *
+glade_app_undo_redo_button_new (GladeApp *app, gboolean undo)
+{
+	GtkWidget *button;
+	
+	button = gtk_button_new_from_stock ((undo) ? 
+					    GTK_STOCK_UNDO : 
+					    GTK_STOCK_REDO);
+	
+	g_signal_connect_swapped (button, "clicked",
+				  (undo) ? G_CALLBACK (glade_app_command_undo) : 
+					   G_CALLBACK (glade_app_command_redo),
+				  app);
+	
+	if (undo)
+	{
+		app->priv->undo_list = g_list_prepend (app->priv->undo_list, button);
+		g_signal_connect (button, "destroy",
+				  G_CALLBACK (glade_app_undo_button_destroyed),
+				  app);
+	}
+	else
+	{
+		app->priv->redo_list = g_list_prepend (app->priv->redo_list, button);
+		g_signal_connect (button, "destroy",
+				  G_CALLBACK (glade_app_redo_button_destroyed),
+				  app);
+	}
+	
+	glade_app_refresh_undo_redo_button (app, button, undo);
+	
+	return button;
+}
+
+/*
+ * glade_app_undo_button_new:
+ *
+ * Creates a new GtkButton undo widget.
+ * The button will be automatically updated with @app's undo stack.
+ */
+GtkWidget *
+glade_app_undo_button_new (GladeApp *app)
+{
+	return glade_app_undo_redo_button_new (app, TRUE);
+}
+
+/*
+ * glade_app_redo_button_new:
+ *
+ * Creates a new GtkButton redo widget.
+ * The button will be automatically updated with @app's redo stack.
+ */
+GtkWidget *
+glade_app_redo_button_new (GladeApp *app)
+{
+	return glade_app_undo_redo_button_new (app, FALSE);
 }
 
 /* Default application convinience functions */
@@ -1005,7 +1148,16 @@ glade_default_app_selection_changed (void)
 	}
 }
 
+GtkWidget *
+glade_default_app_undo_button_new (void)
+{
+	g_return_val_if_fail (glade_default_app != NULL, NULL);
+	return glade_app_undo_redo_button_new (glade_default_app, TRUE);
+}
 
-
-
-G_END_DECLS
+GtkWidget *
+glade_default_app_redo_button_new (void)
+{
+	g_return_val_if_fail (glade_default_app != NULL, NULL);
+	return glade_app_undo_redo_button_new (glade_default_app, FALSE);
+}
