@@ -15,7 +15,9 @@ LIBGLADEUI_API GType       glade_standard_stock_get_type (void);
 LIBGLADEUI_API GParamSpec *glade_standard_stock_spec     (void);
 LIBGLADEUI_API GParamSpec *glade_standard_int_spec       (void);
 LIBGLADEUI_API GParamSpec *glade_standard_string_spec    (void);
+LIBGLADEUI_API GParamSpec *glade_standard_strv_spec      (void);
 LIBGLADEUI_API GParamSpec *glade_standard_float_spec     (void);
+LIBGLADEUI_API GParamSpec *glade_standard_boolean_spec   (void);
 
 G_END_DECLS
 
