@@ -31,6 +31,7 @@
 #include <libxml/parser.h>
 
 #include "glade-parser.h"
+#include "glade.h"
 
 #define GLADE_NOTE(a,b)
 
@@ -105,6 +106,7 @@ struct _GladeParseState {
 
     enum {PROP_NONE, PROP_WIDGET, PROP_ATK, PROP_CHILD } prop_type;
     gchar *prop_name;
+    gchar *comment;
     gboolean translate_prop;
     gboolean context_prop;
     GArray *props;
@@ -115,38 +117,6 @@ struct _GladeParseState {
     GArray *accels;
 };
 
-static gchar *
-alloc_string(GladeInterface *interface, const gchar *string)
-{
-    gchar *s;
-
-    s = g_hash_table_lookup(interface->strings, string);
-    if (!s) {
-	s = g_strdup(string);
-	g_hash_table_insert(interface->strings, s, s);
-    }
-    return s;
-}
-
-static gchar *
-alloc_propname(GladeInterface *interface, const gchar *string)
-{
-    static GString *norm_str;
-    gint i;
-
-    if (!norm_str)
-	norm_str = g_string_new_len(NULL, 64);
-
-    /* assign the string to norm_str */
-    g_string_assign(norm_str, string);
-    /* convert all dashes to underscores */
-    for (i = 0; i < norm_str->len; i++)
-	if (norm_str->str[i] == '-')
-	    norm_str->str[i] = '_';
-
-    return alloc_string(interface, norm_str->str);
-}
-
 static GladeWidgetInfo *
 create_widget_info(GladeInterface *interface, const xmlChar **attrs)
 {
@@ -463,16 +433,12 @@ glade_parser_start_document(GladeParseState *state)
     state->widget_depth = 0;
     state->content = g_string_sized_new(128);
 
-    state->interface = g_new0(GladeInterface, 1);
-    state->interface->names = g_hash_table_new(g_str_hash, g_str_equal);
-    state->interface->strings = g_hash_table_new_full(g_str_hash,
-						      g_str_equal,
-						      (GDestroyNotify)g_free,
-						      NULL);
+    state->interface = glade_interface_new ();
     state->widget = NULL;
 
     state->prop_type = PROP_NONE;
     state->prop_name = NULL;
+    state->comment = NULL;
     state->translate_prop = FALSE;
     state->props = NULL;
 
@@ -551,6 +517,7 @@ glade_parser_start_element(GladeParseState *state,
 	    state->widget_depth++;
 	    state->prop_type = PROP_NONE;
 	    state->prop_name = NULL;
+	    state->comment = NULL;
 	    state->props = NULL;
 	    state->signals = NULL;
 	    state->accels = NULL;
@@ -586,6 +553,9 @@ glade_parser_start_element(GladeParseState *state,
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
+		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
+		    state->comment = alloc_propname(state->interface, 
+						     CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("agent")))
 		    bad_agent = xmlStrcmp(attrs[i], BAD_CAST("libglade")) != 0;
 		else
@@ -644,6 +614,9 @@ glade_parser_start_element(GladeParseState *state,
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
+		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
+		    state->comment = alloc_propname(state->interface, 
+						     CAST_BAD(attrs[i+1]));
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
 			      attrs[i]);
@@ -753,6 +726,7 @@ glade_parser_start_element(GladeParseState *state,
 	    state->widget_depth++;
 	    state->prop_type = PROP_NONE;
 	    state->prop_name = NULL;
+	    state->comment = NULL;
 	    state->props = NULL;
 	    state->signals = NULL;
 	    state->accels = NULL;
@@ -796,6 +770,9 @@ glade_parser_start_element(GladeParseState *state,
 		    state->translate_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("context")))
 		    state->context_prop = !xmlStrcmp(attrs[i+1], BAD_CAST("yes"));
+		else if (!xmlStrcmp(attrs[i], BAD_CAST("comments")))
+		    state->comment = alloc_propname(state->interface, 
+						     CAST_BAD(attrs[i+1]));
 		else if (!xmlStrcmp(attrs[i], BAD_CAST("agent")))
 		    bad_agent = xmlStrcmp(attrs[i], BAD_CAST("libglade")) != 0;
 		else
@@ -908,20 +885,16 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	    g_warning("should find </property> here.  Found </%s>", name);
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladePropInfo));
-	prop.name = state->prop_name;
- 	if (state->translate_prop && state->content->str[0] != '\0') {
-	    if (state->context_prop) 
-		prop.value = alloc_string(state->interface,
-					  g_strip_context(state->content->str,
-							   dgettext(state->domain, state->content->str)));
-	    else
-		prop.value = alloc_string(state->interface,
-					  dgettext(state->domain, state->content->str));
- 	} else {
- 	    prop.value = alloc_string(state->interface, state->content->str);
-  	}
+
+	prop.name         = state->prop_name;
+	prop.has_context  = state->context_prop;
+	prop.translatable = state->translate_prop;
+	prop.comment      = state->comment;
+	prop.value        = alloc_string(state->interface, state->content->str);
+
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
+	state->comment   = NULL;
 	state->state = PARSER_WIDGET;
 	break;
     case PARSER_WIDGET_ATK:
@@ -935,20 +908,16 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	    g_warning("should find </atkproperty> here.  Found </%s>", name);
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladePropInfo));
-	prop.name = state->prop_name;
- 	if (state->translate_prop && state->content->str[0] != '\0') {
-	    if (state->context_prop) 
-		prop.value = alloc_string(state->interface,
-					  g_strip_context(state->content->str,
-							   dgettext(state->domain, state->content->str)));
-	    else
-		prop.value = alloc_string(state->interface,
-					  dgettext(state->domain, state->content->str));
- 	} else {
- 	    prop.value = alloc_string(state->interface, state->content->str);
-  	}
+	prop.name         = state->prop_name;
+	prop.has_context  = state->context_prop;
+	prop.translatable = state->translate_prop;
+	prop.comment      = state->comment;
+	prop.value        = alloc_string(state->interface, state->content->str);
+
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
+	state->comment   = NULL;
+
 	state->state = PARSER_WIDGET_ATK;
 	break;
     case PARSER_WIDGET_ATK_ACTION:
@@ -1002,20 +971,17 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	    g_warning("should find </property> here.  Found </%s>", name);
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladePropInfo));
-	prop.name = state->prop_name;
- 	if (state->translate_prop && state->content->str[0] != '\0') {
-	    if (state->context_prop) 
-		prop.value = alloc_string(state->interface,
-					  g_strip_context(state->content->str,
-							   dgettext(state->domain, state->content->str)));
-	    else
-		prop.value = alloc_string(state->interface,
-					  dgettext(state->domain, state->content->str));
- 	} else {
- 	    prop.value = alloc_string(state->interface, state->content->str);
-  	}
+
+	prop.name         = state->prop_name;
+	prop.has_context  = state->context_prop;
+	prop.translatable = state->translate_prop;
+	prop.comment      = state->comment;
+	prop.value        = alloc_string(state->interface, state->content->str);
+
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
+	state->comment   = NULL;
+
 	state->state = PARSER_WIDGET_CHILD_PACKING;
 	break;
     case PARSER_WIDGET_CHILD_AFTER_PACKING:
@@ -1142,6 +1108,24 @@ widget_info_free(GladeWidgetInfo *info)
     g_free(info);
 }
 
+/**
+ * glade_interface_new
+ *
+ * Returns a newly allocated GladeInterface.
+ */
+GladeInterface *
+glade_interface_new ()
+{
+	GladeInterface *interface;
+	interface = g_new0 (GladeInterface, 1);
+	interface->names = g_hash_table_new (g_str_hash, g_str_equal);
+	interface->strings = g_hash_table_new_full (g_str_hash,
+						    g_str_equal,
+						    (GDestroyNotify)g_free,
+						    NULL);
+	return interface;
+}
+
 /**
  * glade_interface_destroy
  * @interface: the GladeInterface structure.
@@ -1191,7 +1175,9 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
     GladeParseState state = { 0 };
 
     if (!g_file_test(file, G_FILE_TEST_IS_REGULAR)) {
-	g_warning("could not find glade file '%s'", file);
+	glade_util_ui_message (glade_default_app_get_window (), 
+			       GLADE_UI_ERROR,
+			       _("Could not find glade file %s"), file);
 	return NULL;
     }
 
@@ -1202,13 +1188,17 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 	state.domain = textdomain(NULL);
 
     if (xmlSAXUserParseFile(&glade_parser, &state, file) < 0) {
-	g_warning("document not well formed");
+	glade_util_ui_message (glade_default_app_get_window (), 
+			       GLADE_UI_ERROR,
+			       _("Errors parsing glade file %s"), file);
 	if (state.interface)
 	    glade_interface_destroy (state.interface);
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
-	g_warning("did not finish in PARSER_FINISH state");
+	glade_util_ui_message (glade_default_app_get_window (), 
+			       GLADE_UI_ERROR,
+			       _("Errors parsing glade file %s"), file);
 	if (state.interface)
 	    glade_interface_destroy(state.interface);
 	return NULL;
@@ -1283,6 +1273,13 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 	    xmlNodeAddContent(widget, BAD_CAST("  "));
 	node = xmlNewNode(NULL, BAD_CAST("property"));
 	xmlSetProp(node, BAD_CAST("name"), BAD_CAST(info->properties[i].name));
+	if (info->properties[i].translatable)
+	    xmlSetProp(node, BAD_CAST("translatable"), BAD_CAST("yes"));
+	if (info->properties[i].has_context)
+	    xmlSetProp(node, BAD_CAST("context"), BAD_CAST("yes"));
+	if (info->properties[i].comment)
+	    xmlSetProp(node, BAD_CAST("comments"), 
+		       BAD_CAST(info->properties[i].comment));
 	xmlNodeSetContent(node, BAD_CAST(info->properties[i].value));
 	xmlAddChild(widget, node);
 	xmlNodeAddContent(widget, BAD_CAST("\n"));
@@ -1305,6 +1302,13 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 		xmlNodeAddContent(atk, BAD_CAST("  "));
 	    node = xmlNewNode(NULL, BAD_CAST("property"));
 	    xmlSetProp(node, BAD_CAST("name"), BAD_CAST(info->atk_props[i].name));
+	    if (info->atk_props[i].translatable)
+		xmlSetProp(node, BAD_CAST("translatable"), BAD_CAST("yes"));
+	    if (info->atk_props[i].has_context)
+		xmlSetProp(node, BAD_CAST("context"), BAD_CAST("yes"));
+	    if (info->atk_props[i].comment)
+		xmlSetProp(node, BAD_CAST("comments"), 
+			   BAD_CAST(info->atk_props[i].comment));
 	    xmlNodeSetContent(node, BAD_CAST(info->atk_props[i].value));
 	    xmlAddChild(atk, node);
 	    xmlNodeAddContent(atk, BAD_CAST("\n"));
@@ -1380,6 +1384,13 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
 		    xmlNodeAddContent(packing, BAD_CAST("  "));
 		node = xmlNewNode(NULL, BAD_CAST("property"));
 		xmlSetProp(node, BAD_CAST("name"), BAD_CAST(childinfo->properties[k].name));
+		if (childinfo->properties[k].translatable)
+		    xmlSetProp(node, BAD_CAST("translatable"), BAD_CAST("yes"));
+		if (childinfo->properties[k].has_context)
+		    xmlSetProp(node, BAD_CAST("context"), BAD_CAST("yes"));
+		if (childinfo->properties[k].comment)
+		    xmlSetProp(node, BAD_CAST("comments"), 
+			       BAD_CAST(childinfo->properties[k].comment));
 		xmlNodeSetContent(node, BAD_CAST(childinfo->properties[k].value));
 		xmlAddChild(packing, node);
 		xmlNodeAddContent(packing, BAD_CAST("\n"));
@@ -1500,9 +1511,30 @@ glade_interface_dump_full(GladeInterface *interface, const gchar *filename, GErr
 void
 glade_interface_dump(GladeInterface *interface, const gchar *filename)
 {
-    glade_interface_dump_full(interface, filename, NULL);
+	glade_interface_dump_full(interface, filename, NULL);
 }
 
+
+G_CONST_RETURN gchar *
+glade_parser_pvalue_from_winfo (GladeWidgetInfo *winfo,
+				const gchar     *pname)
+{
+	gchar *dup_name = g_strdup (pname);
+	gint i;
+
+	/* Make '_' & '-' synonymous here for convenience.
+	 */
+	glade_util_replace (dup_name, '-', '_');
+	for (i = 0; i < winfo->n_properties; i++)
+	{
+		if (!strcmp (pname, winfo->properties[i].name) ||
+		    !strcmp (dup_name, winfo->properties[i].name))
+			return winfo->properties[i].value;
+	}
+	return NULL;
+}
+
+
 #if 0
 int
 main(int argc, char **argv) {
