@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors:
  *   Chema Celorio <chema@celorio.com>
@@ -35,7 +35,7 @@
 #include "glade-parameter.h"
 #include "glade-property.h"
 #include "glade-property-class.h"
-#include "glade-editor.h"
+#include "glade-editor-property.h"
 #include "glade-debug.h"
 
 /**
@@ -69,8 +69,8 @@ glade_property_class_new (void)
 	property_class->visible = TRUE;
 	property_class->save = TRUE;
 	property_class->ignore = FALSE;
+	property_class->resource = FALSE;
 	property_class->translatable = TRUE;
-
 	return property_class;
 }
 
@@ -343,7 +343,13 @@ glade_property_class_make_string_from_gvalue (GladePropertyClass *property_class
 	else if (G_IS_PARAM_SPEC_DOUBLE(property_class->pspec))
 		string = g_strdup_printf ("%g", g_value_get_double (value));
 	else if (G_IS_PARAM_SPEC_STRING(property_class->pspec))
-		string = g_value_dup_string (value);
+	{
+		if (property_class->resource && g_value_get_string (value) != NULL)
+			string = g_path_get_basename 
+				(g_value_get_string (value));
+		else
+			string = g_value_dup_string (value);
+	}
 	else if (G_IS_PARAM_SPEC_CHAR(property_class->pspec))
 		string = g_strdup_printf ("%c", g_value_get_char (value));
 	else if (G_IS_PARAM_SPEC_UCHAR(property_class->pspec))
@@ -357,6 +363,30 @@ glade_property_class_make_string_from_gvalue (GladePropertyClass *property_class
 	else if (G_IS_PARAM_SPEC_BOOLEAN(property_class->pspec))
 		string = g_strdup_printf ("%s", g_value_get_boolean (value) ?
 					  GLADE_TAG_TRUE : GLADE_TAG_FALSE);
+	else if (G_IS_PARAM_SPEC_OBJECT(property_class->pspec))
+	{
+		GObject *object = g_value_get_object (value);
+
+		if (property_class->pspec->value_type == GDK_TYPE_PIXBUF)
+		{
+			gchar *filename;
+
+			if (object && (filename = g_object_get_data
+				       (object, "GladeFileName")) != NULL)
+				string = g_path_get_basename (filename);
+		}
+		else
+		{
+			GladeWidget *gwidget;
+			if (object)
+			{
+				if ((gwidget = glade_widget_get_from_gobject (object)) != NULL)
+					string = g_strdup (gwidget->name);
+				else
+					g_critical ("Object type property refers to an object outside the project");
+			}
+		}
+	}
 	else
 		g_critical ("Unsupported pspec type %s",
 			    g_type_name(G_PARAM_SPEC_TYPE (property_class->pspec)));
@@ -448,9 +478,6 @@ glade_property_class_make_flags_from_string (GType type, const char *string)
 }
 
 /* This is copied exactly from libglade. I've just renamed the function.
- *
- * TODO: Expose function from libglade and use the one in libglade (once
- * Ivan Wongs libglade patch gets in...).
  */
 static gint
 glade_property_class_make_enum_from_string (GType type, const char *string)
@@ -477,19 +504,21 @@ glade_property_class_make_enum_from_string (GType type, const char *string)
 
 /**
  * glade_property_class_make_gvalue_from_string:
- * @property_class:
- * @string:
+ * @property_class: A #GladePropertyClass
+ * @string: a string representation of this property
+ * @project: the glade project that the associated property
+ *           belongs to.
  *
- * TODO: write me
- *
- * Returns:
+ * Returns: A #GValue created based on the @property_class
+ *          and @string criteria.
  */
 GValue *
 glade_property_class_make_gvalue_from_string (GladePropertyClass *property_class,
-					      const gchar *string)
+					      const gchar        *string,
+					      GladeProject       *project)
 {
 	GValue    *value = g_new0 (GValue, 1);
-	gchar    **strv;
+	gchar    **strv, *fullpath;
 	GdkColor   color = { 0, };
 
 	g_value_init (value, property_class->pspec->value_type);
@@ -544,7 +573,20 @@ glade_property_class_make_gvalue_from_string (GladePropertyClass *property_class
 	else if (G_IS_PARAM_SPEC_DOUBLE(property_class->pspec))
 		g_value_set_double (value, (float) atof (string));
 	else if (G_IS_PARAM_SPEC_STRING(property_class->pspec))
-		g_value_set_string (value, string);
+	{
+		/* This can be called when loading defaults from
+		 * catalog files... it wont happen and we cant do
+		 * anything for it.
+		 */
+		if (property_class->resource && project)
+		{
+			fullpath = g_build_filename
+				(project->path, string, NULL);
+			g_value_set_string (value, fullpath);
+			g_free (fullpath);
+		}
+		else g_value_set_string (value, string);
+	}
 	else if (G_IS_PARAM_SPEC_CHAR(property_class->pspec))
 		g_value_set_char (value, string[0]);
 	else if (G_IS_PARAM_SPEC_UCHAR(property_class->pspec))
@@ -558,6 +600,41 @@ glade_property_class_make_gvalue_from_string (GladePropertyClass *property_class
 		else
 			g_value_set_boolean (value, FALSE);
 	}
+	else if (G_IS_PARAM_SPEC_OBJECT(property_class->pspec))
+	{
+		if (property_class->pspec->value_type == GDK_TYPE_PIXBUF && project)
+		{
+			GdkPixbuf *pixbuf;
+
+			if (string)
+			{
+				fullpath = 
+					glade_project_resource_fullpath (project,
+									 string);
+				if ((pixbuf = gdk_pixbuf_new_from_file
+				     (fullpath, NULL)) != NULL)
+				{
+					g_object_set_data_full (G_OBJECT(pixbuf), 
+								"GladeFileName",
+								g_strdup (string),
+								g_free);
+					g_value_take_object (value, G_OBJECT(pixbuf));
+				}
+				g_free (fullpath);
+			}
+			else
+				g_value_set_object (value, NULL);
+		}
+		else
+		{
+			GladeWidget *gwidget;
+			if (string && (gwidget = glade_project_get_widget_by_name 
+				       (project, string)) != NULL)
+				g_value_set_object (value, gwidget->object);
+			else
+				g_value_set_object (value, NULL);
+		}
+	}
 	else
 		g_critical ("Unsupported pspec type %s",
 			    g_type_name(G_PARAM_SPEC_TYPE (property_class->pspec)));
@@ -565,6 +642,110 @@ glade_property_class_make_gvalue_from_string (GladePropertyClass *property_class
 	return value;
 }
 
+GValue *
+glade_property_class_make_gvalue_from_vl (GladePropertyClass  *class,
+					  va_list              vl)
+{
+	GValue   *value;
+
+	g_return_val_if_fail (class != NULL, NULL);
+
+	value = g_new0 (GValue, 1);
+	g_value_init (value, class->pspec->value_type);
+	
+	if (G_IS_PARAM_SPEC_ENUM(class->pspec))
+		g_value_set_enum (value, va_arg (vl, gint));
+	else if (G_IS_PARAM_SPEC_FLAGS(class->pspec))
+		g_value_set_flags (value, va_arg (vl, gint));
+	else if (G_IS_PARAM_SPEC_INT(class->pspec))
+		g_value_set_int (value, va_arg (vl, gint));
+	else if (G_IS_PARAM_SPEC_UINT(class->pspec))
+		g_value_set_uint (value, va_arg (vl, guint));
+	else if (G_IS_PARAM_SPEC_LONG(class->pspec))
+		g_value_set_long (value, va_arg (vl, glong));
+	else if (G_IS_PARAM_SPEC_ULONG(class->pspec))
+		g_value_set_ulong (value, va_arg (vl, gulong));
+	else if (G_IS_PARAM_SPEC_INT64(class->pspec))
+		g_value_set_int64 (value, va_arg (vl, gint64));
+	else if (G_IS_PARAM_SPEC_UINT64(class->pspec))
+		g_value_set_uint64 (value, va_arg (vl, guint64));
+	else if (G_IS_PARAM_SPEC_FLOAT(class->pspec))
+		g_value_set_float (value, (gfloat)va_arg (vl, gdouble));
+	else if (G_IS_PARAM_SPEC_DOUBLE(class->pspec))
+		g_value_set_double (value, va_arg (vl, gdouble));
+	else if (G_IS_PARAM_SPEC_STRING(class->pspec))
+		g_value_set_string (value, va_arg (vl, gchar *));
+	else if (G_IS_PARAM_SPEC_CHAR(class->pspec))
+		g_value_set_char (value, (gchar)va_arg (vl, gint));
+	else if (G_IS_PARAM_SPEC_UCHAR(class->pspec))
+		g_value_set_uchar (value, (guchar)va_arg (vl, guint));
+	else if (G_IS_PARAM_SPEC_UNICHAR(class->pspec))
+		g_value_set_uint (value, va_arg (vl, gunichar));
+	else if (G_IS_PARAM_SPEC_BOOLEAN(class->pspec))
+		g_value_set_boolean (value, va_arg (vl, gboolean));
+	else if (G_IS_PARAM_SPEC_OBJECT(class->pspec))
+		g_value_set_object (value, va_arg (vl, gpointer));
+	else if (G_IS_PARAM_SPEC_BOXED(class->pspec))
+		g_value_set_boxed (value, va_arg (vl, gpointer));
+	else
+		g_critical ("Unsupported pspec type %s",
+			    g_type_name(class->pspec->value_type));
+	
+	return value;
+}
+
+
+void
+glade_property_class_set_vl_from_gvalue (GladePropertyClass  *class,
+					 GValue              *value,
+					 va_list              vl)
+{
+	g_return_if_fail (class != NULL);
+	g_return_if_fail (value != NULL);
+
+	/* The argument is a pointer of the specified type, cast the pointer and assign
+	 * the value using the proper g_value_get_ variation.
+	 */
+	if (G_IS_PARAM_SPEC_ENUM(class->pspec))
+		*(gint *)(va_arg (vl, gint *)) = g_value_get_enum (value);
+	else if (G_IS_PARAM_SPEC_FLAGS(class->pspec))
+		*(gint *)(va_arg (vl, gint *)) = g_value_get_flags (value);
+	else if (G_IS_PARAM_SPEC_INT(class->pspec))
+		*(gint *)(va_arg (vl, gint *)) = g_value_get_int (value);
+	else if (G_IS_PARAM_SPEC_UINT(class->pspec))
+		*(guint *)(va_arg (vl, guint *)) = g_value_get_uint (value);
+	else if (G_IS_PARAM_SPEC_LONG(class->pspec))
+		*(glong *)(va_arg (vl, glong *)) = g_value_get_long (value);
+	else if (G_IS_PARAM_SPEC_ULONG(class->pspec))
+		*(gulong *)(va_arg (vl, gulong *)) = g_value_get_ulong (value);
+	else if (G_IS_PARAM_SPEC_INT64(class->pspec))
+		*(gint64 *)(va_arg (vl, gint64 *)) = g_value_get_int64 (value);
+	else if (G_IS_PARAM_SPEC_UINT64(class->pspec))
+		*(guint64 *)(va_arg (vl, guint64 *)) = g_value_get_uint64 (value);
+	else if (G_IS_PARAM_SPEC_FLOAT(class->pspec))
+		*(gfloat *)(va_arg (vl, gdouble *)) = g_value_get_float (value);
+	else if (G_IS_PARAM_SPEC_DOUBLE(class->pspec))
+		*(gdouble *)(va_arg (vl, gdouble *)) = g_value_get_double (value);
+	else if (G_IS_PARAM_SPEC_STRING(class->pspec))
+		*(gchar **)(va_arg (vl, gchar *)) = (gchar *)g_value_get_string (value);
+	else if (G_IS_PARAM_SPEC_CHAR(class->pspec))
+		*(gchar *)(va_arg (vl, gint *)) = g_value_get_char (value);
+	else if (G_IS_PARAM_SPEC_UCHAR(class->pspec))
+		*(guchar *)(va_arg (vl, guint *)) = g_value_get_uchar (value);
+	else if (G_IS_PARAM_SPEC_UNICHAR(class->pspec))
+		*(guint *)(va_arg (vl, gunichar *)) = g_value_get_uint (value);
+	else if (G_IS_PARAM_SPEC_BOOLEAN(class->pspec))
+		*(gboolean *)(va_arg (vl, gboolean *)) = g_value_get_boolean (value);
+	else if (G_IS_PARAM_SPEC_OBJECT(class->pspec))
+		*(gpointer *)(va_arg (vl, gpointer *)) = g_value_get_object (value);
+	else if (G_IS_PARAM_SPEC_BOXED(class->pspec))
+		*(gpointer *)(va_arg (vl, gpointer *)) = g_value_get_boxed (value);
+	else
+		g_critical ("Unsupported pspec type %s",
+			    g_type_name(class->pspec->value_type));
+}
+
+
 /**
  * glade_property_class_new_from_spec:
  * @spec:
@@ -588,7 +769,7 @@ glade_property_class_new_from_spec (GParamSpec *spec)
 
 	/* Register only editable properties.
 	 */
-	if (!glade_editor_editable_property (property_class->pspec))
+	if (!glade_editor_property_supported (property_class->pspec))
 		goto lblError;
 	
 	property_class->id   = g_strdup (spec->name);
@@ -622,18 +803,36 @@ glade_property_class_new_from_spec (GParamSpec *spec)
 
 /**
  * glade_property_class_is_visible:
- * @property_class:
+ * @property_class: A #GladePropertyClass
  *
- * TODO: write me
  *
- * Returns:
+ * Returns: whether or not to show this property in the editor
  */
 gboolean
-glade_property_class_is_visible (GladePropertyClass *property_class)
+glade_property_class_is_visible (GladePropertyClass *class)
 {
-	return property_class->visible;
+	g_return_val_if_fail (GLADE_IS_PROPERTY_CLASS (class), FALSE);
+	return class->visible;
 }
 
+
+/**
+ * glade_property_class_is_object:
+ * @property_class: A #GladePropertyClass
+ *
+ *
+ * Returns: whether or not this is an object property 
+ * that refers to another object in this project.
+ */
+gboolean
+glade_property_class_is_object (GladePropertyClass  *class)
+{
+	g_return_val_if_fail (GLADE_IS_PROPERTY_CLASS (class), FALSE);
+	return (G_IS_PARAM_SPEC_OBJECT(class->pspec) &&
+		class->pspec->value_type != GDK_TYPE_PIXBUF);
+}
+
+
 /**
  * glade_property_class_get_displayable_value:
  * @class: the property class to search in
@@ -647,15 +846,15 @@ gchar *
 glade_property_class_get_displayable_value(GladePropertyClass *class, gint value)
 {
 	gint i, len;
-	GArray *disp_val=class->displayable_values;
+	GArray *disp_val = class->displayable_values;
 
 	if (disp_val == NULL) return NULL;
 	
-	len=disp_val->len;
+	len = disp_val->len;
 	
 	for (i = 0; i < len; i++)
-		if (g_array_index(disp_val, GEnumValue, i).value == value)
-			return g_array_index(disp_val, GEnumValue, i).value_name;
+		if (g_array_index (disp_val, GEnumValue, i).value == value)
+			return g_array_index (disp_val, GEnumValue, i).value_name;
 
 	return NULL;
 }
@@ -669,13 +868,16 @@ glade_property_class_get_displayable_value(GladePropertyClass *class, gint value
  * Returns: a (GArray *) of GEnumValue of the overridden fields.
  */
 static GArray *
-gpc_get_displayable_values_from_node (GladeXmlNode *node, GEnumValue *values, gint n_values)
+gpc_get_displayable_values_from_node (GladeXmlNode *node, 
+				      GEnumValue   *values, 
+				      gint          n_values,
+				      const gchar  *domain)
 {
 	GArray *array;
 	GladeXmlNode *child;
 	
-	child = glade_xml_search_child (node, GLADE_TAG_VALUE);
-	if (child == NULL) return NULL;
+	if ((child = glade_xml_search_child (node, GLADE_TAG_VALUE)) == NULL)
+		return NULL;
 	
 	array = g_array_new (FALSE, TRUE, sizeof(GEnumValue));
 
@@ -689,16 +891,29 @@ gpc_get_displayable_values_from_node (GladeXmlNode *node, GEnumValue *values, gi
 		id = glade_xml_get_property_string_required (child, GLADE_TAG_ID, NULL);
 		name = glade_xml_get_property_string (child, GLADE_TAG_NAME);
 		nick = glade_xml_get_property_string (child, GLADE_TAG_NICK);
-		
+
 		for(i=0; i < n_values; i++)
 		{
 			if(strcmp (id, values[i].value_name) == 0)
 			{
 				val=values[i];
 				
-				if(name) val.value_name = name;
-				if(nick) val.value_nick = nick;
-				
+				/* Tedious memory handleing; if gettext doesn't return
+				 * a translation, dont free the untranslated string.
+				 */
+				if(name) 
+				{
+					val.value_name = dgettext (domain, name);
+					if (name != val.value_name)
+						g_free (name);
+				}
+				if(nick)
+				{
+					val.value_nick = dgettext (domain, nick);
+					if (nick != val.value_nick)
+						g_free (nick);
+				}
+
 				g_array_append_val(array, val);
 				break;
 			}
@@ -788,8 +1003,10 @@ glade_property_class_make_adjustment (GladePropertyClass *property_class)
 /**
  * glade_property_class_update_from_node:
  * @node: the <property> node
- * @widget_class: the widget class
+ * @module: a #GModule to lookup symbols from the plugin
+ * @object_type: the #GType of the owning object
  * @property_class: a pointer to the property class
+ * @domain: the domain to translate catalog strings from.
  *
  * Updates the @property_class with the contents of the node in the xml
  * file. Only the values found in the xml file are overridden.
@@ -800,8 +1017,9 @@ glade_property_class_make_adjustment (GladePropertyClass *property_class)
 gboolean
 glade_property_class_update_from_node (GladeXmlNode        *node,
 				       GModule             *module,
-				       GType                widget_type,
-				       GladePropertyClass **property_class)
+				       GType                object_type,
+				       GladePropertyClass **property_class,
+				       const gchar         *domain)
 {
 	GladePropertyClass *class;
 	gchar *buff;
@@ -838,6 +1056,11 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 		/* ... get the tooltip from the pspec ... */
 		if ((class->pspec = glade_utils_get_pspec_from_funcname (buff)) != NULL)
 		{
+			/* Make sure we can tell properties apart by there 
+			 * owning class.
+			 */
+			class->pspec->owner_type = object_type;
+
 			if (class->tooltip) g_free (class->tooltip);
 			if (class->name)    g_free (class->name);
 			
@@ -876,7 +1099,7 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 			g_value_unset (class->def);
 			g_free (class->def);
 		}
-		class->def = glade_property_class_make_gvalue_from_string (class, buff);
+		class->def = glade_property_class_make_gvalue_from_string (class, buff, NULL);
 		g_free (buff);
 	}
 
@@ -885,7 +1108,7 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	if (buff)
 	{
 		g_free (class->name);
-		class->name = buff;
+		class->name = g_strdup (dgettext (domain, buff));
 	}
 	
 	/* ...and the tooltip */
@@ -893,7 +1116,7 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	if (buff)
 	{
 		g_free (class->tooltip);
-		class->tooltip = buff;
+		class->tooltip = g_strdup (dgettext (domain, buff));
 	}
 
 	/* If this property's value is an enumeration then we try to get the displayable values */
@@ -903,7 +1126,8 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 		
 		child = glade_xml_search_child (node, GLADE_TAG_DISPLAYABLE_VALUES);
 		if (child)
-			class->displayable_values = gpc_get_displayable_values_from_node(child, eclass->values, eclass->n_values);
+			class->displayable_values = gpc_get_displayable_values_from_node
+				(child, eclass->values, eclass->n_values, domain);
 		
 		g_type_class_unref(eclass);
 	}
@@ -915,7 +1139,8 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 		
 		child = glade_xml_search_child (node, GLADE_TAG_DISPLAYABLE_VALUES);
 		if (child)
-			class->displayable_values = gpc_get_displayable_values_from_node(child, (GEnumValue*)fclass->values, fclass->n_values);
+			class->displayable_values = gpc_get_displayable_values_from_node
+				(child, (GEnumValue*)fclass->values, fclass->n_values, domain);
 		
 		g_type_class_unref(fclass);
 	}
@@ -935,12 +1160,18 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	class->translatable = glade_xml_get_property_boolean (node, GLADE_TAG_TRANSLATABLE, TRUE);
 
 	/* common, optional, etc */
-	class->common        = glade_xml_get_property_boolean (node, GLADE_TAG_COMMON,         class->common);
-	class->optional      = glade_xml_get_property_boolean (node, GLADE_TAG_OPTIONAL,       class->optional);
-	class->query         = glade_xml_get_property_boolean (node, GLADE_TAG_QUERY,          class->query);
-	class->save          = glade_xml_get_property_boolean (node, GLADE_TAG_SAVE,           class->save);
-	class->visible       = glade_xml_get_property_boolean (node, GLADE_TAG_VISIBLE,        class->visible);
-	class->ignore        = glade_xml_get_property_boolean (node, GLADE_TAG_IGNORE,         class->ignore);
+	class->common   = glade_xml_get_property_boolean (node, GLADE_TAG_COMMON,   class->common);
+	class->optional = glade_xml_get_property_boolean (node, GLADE_TAG_OPTIONAL, class->optional);
+	class->query    = glade_xml_get_property_boolean (node, GLADE_TAG_QUERY,    class->query);
+	class->save     = glade_xml_get_property_boolean (node, GLADE_TAG_SAVE,     class->save);
+	class->visible  = glade_xml_get_property_boolean (node, GLADE_TAG_VISIBLE,  class->visible);
+	class->ignore   = glade_xml_get_property_boolean (node, GLADE_TAG_IGNORE,   class->ignore);
+	class->resource = glade_xml_get_property_boolean (node, GLADE_TAG_RESOURCE, class->resource);
+
+	/* Special case pixbuf here.
+	 */
+	if (class->pspec->value_type == GDK_TYPE_PIXBUF)
+		class->resource = TRUE;
 	
 	if (class->optional)
 		class->optional_default =
@@ -964,14 +1195,14 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 			g_warning (_("The property [%s] of the widget's class [%s] "
 				     "needs a special \"set\" function, but there is "
 				     "no library associated to this widget's class."),
-				   class->name, g_type_name (widget_type));
+				   class->name, g_type_name (object_type));
 
 		if (!g_module_symbol (module, symbol_name, (gpointer *)
 				      &class->set_function))
 			g_warning (_("Unable to get the \"set\" function [%s] of the "
 				     "property [%s] of the widget's class [%s] from "
 				     "the module [%s]: %s"),
-				   symbol_name, class->name, g_type_name (widget_type),
+				   symbol_name, class->name, g_type_name (object_type),
 				   g_module_name (module), g_module_error ());
 		g_free (symbol_name);
 	}
@@ -988,14 +1219,14 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 			g_warning (_("The property [%s] of the widget's class [%s] needs a "
 				     "special \"get\" function, but there is no library "
 				     "associated to this widget's class."),
-				   class->name, g_type_name (widget_type));
+				   class->name, g_type_name (object_type));
 
 		if (!g_module_symbol(module, symbol_name,
 				     (gpointer *) &class->get_function))
 			g_warning (_("Unable to get the \"get\" function [%s] of the "
 				     "property [%s] of the widget's class [%s] from the "
 				     "module [%s]: %s"),
-				   symbol_name, class->name, g_type_name (widget_type),
+				   symbol_name, class->name, g_type_name (object_type),
 				   g_module_name (module), g_module_error ());
 		g_free (symbol_name);
 	}
@@ -1009,14 +1240,14 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 			g_warning (_("The property [%s] of the widget's class [%s] needs a "
 				     "special \"get\" function, but there is no library "
 				     "associated to this widget's class."),
-				   class->name, g_type_name (widget_type));
+				   class->name, g_type_name (object_type));
 
 		if (!g_module_symbol(module, symbol_name,
 				     (gpointer *) &class->verify_function))
 			g_warning (_("Unable to get the \"verify\" function [%s] of the "
 				     "property [%s] of the widget's class [%s] from the "
 				     "module [%s]: %s"),
-				   symbol_name, class->name, g_type_name (widget_type),
+				   symbol_name, class->name, g_type_name (object_type),
 				   g_module_name (module), g_module_error ());
 		g_free (symbol_name);
 	}
@@ -1026,3 +1257,26 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 
 	return TRUE;
 }
+
+
+
+/**
+ * glade_property_class_match:
+ *
+ * @class: a #GladePropertyClass
+ * @comp: a #GladePropertyClass
+ *
+ * Returns: whether @class and @comp are a match or not
+ *          (properties in seperate decendant heirarchies that
+ *           have the same name are not matches).
+ */
+gboolean
+glade_property_class_match (GladePropertyClass *class,
+			    GladePropertyClass *comp)
+{
+	g_return_val_if_fail (class != NULL, FALSE);
+	g_return_val_if_fail (comp != NULL, FALSE);
+
+	return (strcmp (class->id, comp->id) == 0 &&
+		class->pspec->owner_type == comp->pspec->owner_type);
+}
